/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file Object.cpp
\author Connor Deakin
\par E-mail: connortdeakin\@gmail.com
\par Project: Mecha
\date 11/09/2016
\brief
  Contains the implementation of functions within the Object class.

\par
  
*/
/*****************************************************************************/

#include <vector>

#include "Error.h"
#include "Component.h"
#include "GameObjectManager.h"
#include "Space.h"

#include "Object.h"


/*****************************************************************************/
/*!
\brief
  The default constructor for the Object class. This will simply set the type
  of the Object to an empty string.
*/
/*****************************************************************************/
Object::Object() : 
_type(""), _parent(nullptr), _active(true), _sprite(nullptr),
_transform()
{}

Object::Object(const std::string & type) :
  _type(type), _parent(nullptr), _active(true), _sprite(nullptr),
  _transform()
{}

/*****************************************************************************/
/*!
\brief
  A constructor for the Object class. This will take the string type of the
  Object as an argument.

\param type
  The type of the game Object that is being created.
*/
/*****************************************************************************/
Object::Object(const std::string & type, Sprite * sprite) : 
_type(type), _parent(nullptr), _active(true), _sprite(sprite), 
_transform()
{}

/*****************************************************************************/
/*!
\brief
  Object constructor for creating an object that has a parent it is attached to.

\param type
  The type of Object being created. All types are stored in a standard string.
\param parent
  The object that this object is a child of.
*/
/*****************************************************************************/
Object::Object(const std::string & type, Object * parent, Sprite * sprite) : 
 _type(type), _parent(parent), _active(true), _sprite(sprite), 
 _transform(parent->_transform)
{
  _parent->_children.push_back(this);
}

/*****************************************************************************/
/*!
\brief
  Copy constructor for Objects. This is a deep copy, so every component
  on the original will be copied over.

\param other
  This is the object to create the new object from.
*/
/*****************************************************************************/
Object::Object(const Object & other) : 
_type(other._type), _parent(nullptr), _active(true)
{
  // This is a deep copy, so we need to loop through and copy the components over
  for each (Component * component in other._components)
  {
    Add(new Component(*component, this));
  }
}

/*****************************************************************************/
/*!
\brief
  The destructor for the Object class. This will go through the entire vector
  of Components and delete them from memory.
*/
/*****************************************************************************/
Object::~Object()
{
  std::vector<Component *>::iterator it;
  for(it = _components.begin(); it != _components.end(); ++it)
    delete (*it);
  _active = false;
}

/*****************************************************************************/
/*!
\brief
  This will call the update for each of the Components that are withing the
  Object's _components vector.
*/
/*****************************************************************************/
void Object::Update()
{
  std::vector<Component *>::iterator it;
  for (it = _components.begin(); it != _components.end(); ++it)
  {
    (*it)->Update();
    if (!_active)
      break;
  }
}

/*****************************************************************************/
/*!
\brief
  Change this Object's parent to a different Object.

\par Important Notes
  - Use nullptr to remove the parent.

\param new_parent
  The Object's new parent.
*/
/*****************************************************************************/
void Object::ChangeParent(Object * new_parent)
{
  // make sure that this object does not have a parent
  // relationship with self
  Object * upper_parent = new_parent;
  while (upper_parent) {
    if (upper_parent == this) {
      Error error("Object.cpp", "ChangeParent");
      error.Add("Infinite parent loop created by this relationship"); 
      throw(error);
    }
    upper_parent = upper_parent->_parent;
  }
  // set new parent
  if (_parent)
    _parent->RemoveChild(this);
  _parent = new_parent;
  _parent->_children.push_back(this);
  // set transform relationship
  _transform.ChangeParent(&new_parent->_transform);
}

/*****************************************************************************/
/*!
\brief
  Add a child to this Object.

\param new_child
  This Objects new child.
*/
/*****************************************************************************/
void Object::AddChild(Object * new_child)
{
  new_child->ChangeParent(this);
}

/*****************************************************************************/
/*!
\brief
  Removes a child from this Object.

\param child
  The child to be removed.
*/
/*****************************************************************************/
void Object::RemoveChild(Object * child)
{
  // find child and erase
  std::vector<Object *>::iterator it = _children.begin();
  std::vector<Object *>::iterator it_e = _children.end();
  for (; it != it_e; ++it) {
    if (*it == child) {
      (*it)->_parent = nullptr;
      _children.erase(it);
      return;
    }
  }
  // child not found
  Error error("Object.cpp", "RemoveChild");
  error.Add("Child Object was not found among this Object's childrent.");
  throw(error);
  // remove child from transform
  _transform.RemoveChild(&child->_transform);
}

/*****************************************************************************/
/*!
\brief
  Set the Sprite that the Object is drawn with.

\param sprite
  The Object new sprite.
*/
/*****************************************************************************/
void Object::SetSprite(Sprite * sprite)
{
  _sprite = sprite;
}

/*****************************************************************************/
/*!
\brief
  This is used to add a Component to the game Object. It will be added to the
  end of the _components vector.

\param new_component
  A pointer to the new Component being added to the game Object. All new
  components must be dynamically allocated using new.
*/
/*****************************************************************************/
Component * Object::Add(Component * new_component)
{
  new_component->SetOwner(this);
  _components.push_back(new_component);
  return new_component;
}

/*****************************************************************************/
/*!
\brief
  Use this to remove a Component with a certain type from the game Object. If
  there are multiple Components of the same type within the _components, the
  Component of the same type closest to the front of the vector will be removed
  first.

\param type
  The type of the Component being removed

\return true: The Component was removed | false: The Component was not removed
*/
/*****************************************************************************/
bool Object::Remove(std::string type)
{
  std::vector<Component *>::iterator it;
  for(it = _components.begin(); it != _components.end(); ++it)
  {
    if(!type.compare((*it)->Type()))
    {
      AbsRemove(*it);
      return true;
    }
  }
  return false;
}

/*****************************************************************************/
/*!
\brief
  Use this to remove a specific Component from a function. Given a pointer to
  the Component to be remove, this function will look through the _components
  vector to find the Component. If the exact Component is found, it will be
  removed from the vector.

\param component
  A pointer to the Component to be removed.

\return true: The Component was removed | false: The Component was not removed
*/
/*****************************************************************************/
bool Object::Remove(Component * component)
{
  std::vector<Component *>::iterator it;
  for(it = _components.begin(); it != _components.end(); ++it)
  {
    if((*it) == component)
    {
      AbsRemove(*it);
      return true;
    }
  }
  return false;
}

/*!
\brief Get the Sprite used for this Object.
\return This Object's Sprite.
*/
Sprite * Object::GetSprite()
{
  return _sprite;
}

/*!
\brief Get the Transform used for this Object.
\return This Object's Transform.
*/
Transform * Object::GetTransform()
{
  return &_transform;
}

/*****************************************************************************/
/*!
\brief
  Get the type of the Object's type that is stored in string data.

\return The Object's type.
*/
/*****************************************************************************/
std::string Object::Type() const
{
  return _type;
}

/*****************************************************************************/
/*!
\brief
  Use this to get a pointer to the parent Object of this Object.

\return A pointer to the Object that this Object is a child of.
*/
/*****************************************************************************/
Object * Object::Parent() const
{
  return _parent;
}

/*****************************************************************************/
/*!
\brief
  This is to be used only within this class. Assuming you have the pointer
  that is stored in the _components vector for a component, that pointer can
  be passed to this function to remove it from the _component's vector. If you
  were to use the same pointer stored in a different memory location, this
  would not work

\param component
  This is the component being deleted from the _components vector. This is a
  reference to a pointer. Read to brief to find out why.
*/
/*****************************************************************************/
void Object::AbsRemove(Component *& component)
{
  delete component;
  component = *(_components.end() - 1);
  _components.pop_back();
}

/*****************************************************************************/
/*!
\brief
  Adds a callback function to the list of callbacks. If there are no other
  functions subscribed to that eventID, it adds that eventID.

\param eventID
  This is the string ID of the event that the callback function will respond to.

\param callbackFunction
  This is the callback function that will handle the event.
*/
/*****************************************************************************/
void Object::SubscribeCallback(std::string eventID, eventFunction callbackFunction)
{
  auto itr = _callbacks.find(eventID);
  if (itr != _callbacks.end())
  {
    (*itr).second.push_back(callbackFunction);
  }
  else
  {
    _callbacks[eventID] = std::list<eventFunction>();
    _callbacks[eventID].push_back(callbackFunction);
  }
}

/*****************************************************************************/
/*!
\brief
  Removes a callback function from the list of callbacks.

\param eventID
  The eventID the function was subscribed to. This is important because the
  functions are sorted by eventID, so having the ID helps me search faster.

\param callbackFunction
  The function you want to remove from the list.
*/
/*****************************************************************************/
void Object::UnsubscribeCallback(std::string eventID, eventFunction callbackFunction)
{
  auto itr = _callbacks.find(eventID);
  if (itr != _callbacks.end())
  {
    auto itr2 = (*itr).second.begin();
    for (; itr2 != (*itr).second.end(); itr2++)
    {
      if ((*itr2).target<void(Event *)>() == callbackFunction.target<void(Event *)>())
        (*itr).second.erase(itr2);
      return;
    }
  }
}

/*****************************************************************************/
/*!
\brief
  Calls all callback functions subscribed to the event ID of the event passed in
  on this object.

\param _event
  Any event object. Note that it will be cast down to base class pointer, so 
  the callback function will have to cast back if there is more data than eventID.
  Please make sure that the event you are passing has the correct eventID.
*/
/*****************************************************************************/
void Object::DispatchEvent(Event * _event)
{
  auto itr = _callbacks.find(_event->_eventID);
  if (itr != _callbacks.end())
  {
    for each (eventFunction callback in (*itr).second)
    {
      callback(_event);
    }
  }
}

/*****************************************************************************/
/*!
\brief
  Dispatches an event to all children of this object, and their children recursively.
  Does not dispatch the event on this object, however.

\param _event
  The event to pass in. Will be cast down to base class pointer. Make sure the ID is correct.
*/
/*****************************************************************************/
void Object::DispatchDown(Event * _event)
{
  for each(Object * child in _children)
  {
    child->DispatchEvent(_event);
    child->DispatchDown(_event);
  }
}

void Object::Init()
{
  for each(Component * component in _components)
  {
    component->Init();
  }
  if(_sprite)
    (**ObjectManager()->Space()).GetRenderer().Add(_sprite, &_transform);
}

GameObjectManager * Object::ObjectManager() const
{
  return _objManager;
}