/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file Object.h
\author Connor Deakin
\par E-mail: connortdeakin\@gmail.com
\par Project: Mecha
\date 11/09/2016
\brief
  Contains the Interface for the Object class, which is used to store game
  objects and the components associated with the object.
*/
/*****************************************************************************/

/*****************************************************************************/
#ifndef OBJECT_H
#define OBJECT_H

#include <string>
#include <vector>
#include <map>
#include <list>

#include "Event.h"
#include "Sprite.h"
#include "Transform.h"
#include "CollisionManager.h"

class GameObjectManager;
class Component;

/*****************************************************************************/
/*!
\class Object
\brief
  The Object class is the class used for all game Objects. When an Object is
  created, Component's can be added to the object in order to create a specific
  game object type.

\bug
    - Finding a component within a derived component class breaks the Find
      function.
*/
/*****************************************************************************/
class Object
{
  public:
    Object();
    Object(const std::string & type);
    Object(const std::string & type, Sprite * sprite);
    Object(const std::string & type, Object * parent, Sprite * sprite);
    Object(const Object & other);
    ~Object();
    void Update();
    void Init();
    void ChangeParent(Object * new_parent);
    void AddChild(Object * new_child);
    void RemoveChild(Object * child);
    void SetSprite(Sprite * new_sprite);
    Component * Add(Component * new_component);
    bool Remove(std::string type);
    bool Remove(Component * component);
    std::string Type() const;
    Object * Parent() const;
    GameObjectManager * ObjectManager() const;
    Sprite * GetSprite();
    Transform * GetTransform();

    template <typename T>
    T * Find();

    void SubscribeCallback(std::string eventID, eventFunction callbackFunction);
    // Note: The reason you need to pass in an eventID is because the functions are sorted by eventID,
    // so passing in an eventID speeds up the search.
    void UnsubscribeCallback(std::string eventID, eventFunction callbackFunction);

    void DispatchEvent(Event * _event);
    void DispatchDown(Event * _event);

  private:
    //! This is the type of the game Object stored in string form.
    std::string _type;
    //! The vector of Component's that a game Object is composed of.
    std::vector<Component *> _components;
    //! The list of children objects on this game object
    std::vector<Object *> _children;
    //! The Object that this Object is parented to.
    Object * _parent;
    //! Tracks whether the Object is active in the game world.
    bool _active;
    //! The GameObjectManager that is Object is stored in.
    GameObjectManager * _objManager;
    //! This map stores several vectors of event callback functions.
    // the functions are sorted by eventID for faster callback.
    std::map<std::string, std::list<eventFunction> > _callbacks;
    //! The sprite for this Object.
    Sprite * _sprite;
    //! The transform of this Object.
    Transform _transform;

    void AbsRemove(Component *& component);

    friend class GameObjectManager;
    friend class ArchetypeManager;
    friend class CollisionManager;
};


#include <iostream>
#include <typeinfo>

// Yes we had to include this in the header file so it could compile correctly

/*****************************************************************************/
/*!
\brief
Use this to find a Component on an Object.

\param type
This is the type of the Component in a string form that is being searched
for.

\return If the Component is found on the Object, a pointer to the Component
will be retuned. If it is not found, a nullptr is returned by the function.
*/
/*****************************************************************************/
template <typename T>
T * Object::Find()
{
  std::string type = typeid(T).name();
  auto bitr = type.begin();
  // Find the part of the type name after the word class
  type = type.substr(strlen("class "));
  std::vector<Component *>::const_iterator it;
  for (it = _components.begin(); it != _components.end(); ++it)
  {
    if (!type.compare((*it)->Type()))
      return static_cast<T *>(*it);
  }
  return nullptr;
}

#endif
/*****************************************************************************/
