/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file Physics.cpp
\author Gabe Miller
\par E-mail: g.miller\@digipen.edu
\par Project: Boat Game
\date 10/06/2016
\brief
This is the generic physics object for the boat game. This class can be
used for any physics object, and should be used for the player, enemies,
and projectiles.

\par
Functions Include:
	-Constructor
	-Destructor
	-ApplyForce
	-GetVelocity
	-SetVelocity
	-SetAngularVelocity
	-Update
*/
/*****************************************************************************/

#include "Physics.h"
#include "Transform.h"
#include "Time.h"

/*****************************************************************************/
/*!
\brief
  Creator for physics component from json file.

\param value
  A json object which contains the values needed to create a physics object.

\return
  Returns a pointer to the newly created component.
*/
/*****************************************************************************/
Component * CreatePhysicsComponent(Json::Value value)
{
  float mass = value["mass"].asFloat();
  Physics::MotionType motionType = Physics::Dynamic;
  std::string motionTypeName = value["motionType"].asString();
  if (motionTypeName == "Kinematic")
  {
    motionType = Physics::Kinematic;
  }
  if (motionTypeName == "Static")
  {
    motionType = Physics::Static;
  }
  Component * newComponent = new Physics(mass, motionType);
  return newComponent;
}

/*****************************************************************************/
/*!
\brief
	The constructor for the physics object. Default ctor is not used.

\param mass
	The mass of the physics object.

\param motionType
	Defines the type of motion the object will have. Dynamic uses full forces
	to calculate acceleration and velocity, while kinematic only uses velocity,
	and static is for static objects with a collider.
*/
/*****************************************************************************/
Physics::Physics(float mass, MotionType motionType) :
  Component("Physics"), _mass(mass), _motionType(motionType),
  _velocity(glm::vec2(0, 0)), _sumForces(glm::vec2(0, 0)), _angularVelocity(0),
  _dragPercentage(0.99f), _angularDrag(0.9999999f)
{

}

/*****************************************************************************/
/*!
\brief
	The destructor for the physics object. Currently empty.
*/
/*****************************************************************************/
Physics::~Physics()
{

}

/*****************************************************************************/
/*!
\brief
	Apply a vector force to the object at center mass.

\param force
	The force to apply.
*/
/*****************************************************************************/
void Physics::ApplyForce(glm::vec2 force)
{
	_sumForces += force;
}

void Physics::ApplyAngularForce(float force)
{
  _sumAngularForce += force;
}

/*****************************************************************************/
/*!
\brief
	Get the velocity value from the physics object.

\return
	The velocity as a 2D Vector.
*/
/*****************************************************************************/
glm::vec2 Physics::GetVelocity()
{
	return _velocity;
}

/*****************************************************************************/
/*!
\brief
	Set the velocity of the physics component directly.

\param velocity
	The value of the velocity to set.
*/
/*****************************************************************************/
void Physics::SetVelocity(glm::vec2 velocity)
{
	_velocity = velocity;
}

/*****************************************************************************/
/*!
\brief
	Set the angular velocity of the physics component directly.

\param velocity
	The value of the velocity to set.
*/
/*****************************************************************************/
void Physics::SetAngularVelocity(float velocity)
{
	_angularVelocity = velocity;
}


void Physics::SetDrag(float drag)
{
  _dragPercentage = drag;
}

void Physics::SetAngularDrag(float drag)
{
  _angularDrag = drag;
}

#include <iostream>
/*****************************************************************************/
/*!
\brief
	Update for physics class. Runs differently depending on motion type.
*/
/*****************************************************************************/
void Physics::Update()
{
  /* TEMPORARY DT UNTIL ACTUAL FRAME TIME UPDATE IS IN */
	if (_motionType == Dynamic)
	{
		// Calculate this frame's acceleration based on the sum of forces for this frame and the mass.
		glm::vec2 acceleration = _sumForces / _mass;
		// Modify the velocity based on this frame's acceleration and frame time.
		_velocity += acceleration * Time::DT();
    _velocity -= _velocity * _dragPercentage * Time::DT();
    float angularAcceleration = _sumAngularForce / _mass;
    _angularVelocity += angularAcceleration * Time::DT();
    _angularVelocity -= _angularDrag * (_angularVelocity * _angularVelocity) * glm::sign(_angularVelocity) * Time::DT();
		// Reset the sum of forces for the next frame.
		_sumForces = glm::vec2(0, 0);
    _sumAngularForce = 0;
	}
	/* Dynamic and kinematic both use this code to update based on velocity. */
	if (_motionType != Static)
	{
    if (_velocity.x != 0 || _velocity.y != 0 || _angularVelocity != 0)
    {
      // Get the transform component.
      Transform * transform = Owner().GetTransform();
      glm::vec3 translation = transform->GetTranslation();
      float rotation = transform->GetRotation();
      translation += glm::vec3(_velocity * Time::DT(), 0);
      // Update the position of the object based on velocity.
      transform->Translation() = translation;
      transform->Rotation() = rotation += _angularVelocity * Time::DT();
    }
	}
}

float Physics::GetAngularVelocity()
{
  return _angularVelocity;
}

float Physics::GetMass()
{
  return _mass;
}

Physics::MotionType Physics::GetMotionType()
{
  return _motionType;
}