/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file ShaderPool.cpp
\author Connor Deakin
\par E-mail: connor.deakin\@digipen.edu
\par Project: Graphics
\date 13/03/2017
\brief
  The implementation of the ShaderPool.
*/
/*****************************************************************************/

#include <iostream>

#include "GraphicPool.h"
#include "Error.h"

#include "ShaderPool.h"

// static initializations
GraphicShader * ShaderPool::_graphicShader = nullptr;
ShaderTexture * ShaderPool::_textureShader = nullptr;
ShaderTextureSheet * ShaderPool::_textureSheetShader = nullptr;
ShaderScreen * ShaderPool::_screenShader = nullptr;
ShaderScreen * ShaderPool::_invertScreenShader = nullptr;
ShaderWater * ShaderPool::_waterShader = nullptr;
bool ShaderPool::_initialized = false;

/*****************************************************************************/
/*!
\brief
  Initializes the ShaderPool by creating all of the unique Shader types.

\par Important Notes
  - The GraphicPool must be initialized before the ShaderPool.
*/
/*****************************************************************************/
void ShaderPool::Initialize()
{
  // check that the GraphicPool is initialized
  if (!GraphicPool::Initialized()) {
    Error error("ShaderPool.cpp", "Initialize");
    error.Add("The GraphicPool must be initialized before the ShaderPool.");
    throw(error);
  }
  // make sure ShaderPool is not initialized
  if (_initialized) {
    Error error("ShaderPool.cpp", "Initialize");
    error.Add("The ShaderPool has already been initialized once before.");
    throw(error);
  }
  // create shader types
  try {
    _graphicShader = new GraphicShader("Shader/graphic.vert",
                                       "Shader/graphic.frag", 
                                       GraphicPool::Default());
    _textureShader = new ShaderTexture("Shader/textureGraphic.vert",
                                              "Shader/textureGraphic.frag",
                                              GraphicPool::Texture());
    _textureSheetShader = new ShaderTextureSheet(GraphicPool::Texture());
    _screenShader = new ShaderScreen("Shader/screen.vert", 
                                     "Shader/screen.frag");
    _invertScreenShader = new ShaderScreen("Shader/screen.vert",
                                           "Shader/screenInvert.frag");
    _waterShader = new ShaderWater();
  }
  catch (const Error & error) {
    std::cout << error;
  }
  // done
  _initialized = true;
}

/*!
\brief Gets the default shader for the DefaultGraphic.
\return The default shader for the DefaultGraphic.
*/
const GraphicShader & ShaderPool::Graphic()
{
  return *_graphicShader;
}

/*!
\brief Gets the default shader for the TextureGraphic.
\return The default shader for the TextureGraphic.
*/
const ShaderTexture & ShaderPool::Texture()
{
  return *_textureShader;
}

/*!
\brief Gets the default shader for a TextureSheet.
\return The default shader for a TextureSheet.
*/
const ShaderTextureSheet & ShaderPool::TextureSheet()
{
  return *_textureSheetShader;
}

/*!
\brief Gets the default shader for displaying the screen.
\return The default shader for displaying the screen.
*/
const ShaderScreen & ShaderPool::Screen()
{
  return *_screenShader;
}

/*!
\brief Gets the shader used for inverting colors when displaying a frame.
\return The shader used for inverting the screen colors when displaying a frame.
*/
const ShaderScreen & ShaderPool::ScreenInvert()
{
  return *_invertScreenShader;
}

/*!
\brief Gets the shader used for rendering water.
\return The shader used for rendering water.
*/
const ShaderWater & ShaderPool::Water()
{
  return *_waterShader;
}

/*****************************************************************************/
/*!
\brief
  Deletes all of the Shaders in the ShaderPool.
*/
/*****************************************************************************/
void ShaderPool::Purge()
{
  // make sure pool is initialized
  if (!_initialized) {
    Error error("ShaderPool.cpp", "Purge");
    error.Add("Pool cannot be Purged if it has not been initialized.");
    throw(error);
  }
  // purge shaders
  try {
    _graphicShader->Purge();
    _textureShader->Purge();
    _textureSheetShader->Purge();
    _screenShader->Purge();
    _invertScreenShader->Purge();
    _waterShader->Purge();
  }
  catch (const Error & error) {
    std::cout << error;
  }
  // de-allocate
  delete _graphicShader;
  delete _textureShader;
  delete _textureSheetShader;
  delete _screenShader;
  delete _invertScreenShader;
  delete _waterShader;
  // done
  _initialized = false;
}