/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file ShaderScreen.cpp
\author Connor Deakin
\par E-mail: connor.deakin\@digipen.edu
\par Project: Graphics
\date 24/03/2016
\brief
  Contains the implementation of Shaders that are used to display framebuffers.
*/
/*****************************************************************************/

#include "OpenGLError.h"
#include "GraphicPool.h"

#include "ShaderScreen.h"

/*****************************************************************************/
/*!
\brief
  Creates the Shader used to display a framebuffer.

\param vert
  The vertex file used for the screen shader.
\param frag
  The fragment file used for the screen shader.
*/
/*****************************************************************************/
ShaderScreen::ShaderScreen(const std::string & vert, const std::string & frag) :
  Shader(vert, frag)
{
  // get uniform and attribute location
  Use();
  _position_A = glGetAttribLocation(ID(), "in_position");
  _texCoord_A = glGetAttribLocation(ID(), "in_texCoord");
  _screen_U = glGetUniformLocation(ID(), "screen");
  // checking that uniforms and attributes were found
  CHECKATTRIBUTE(_position_A, "in_position", 
    _vertexFile.c_str(), _fragmentFile.c_str());
  CHECKATTRIBUTE(_texCoord_A, "in_texCoord",
    _vertexFile.c_str(), _fragmentFile.c_str());
  CHECKUNIFORM(_screen_U, "screen",
    _vertexFile.c_str(), _fragmentFile.c_str());
  // error check
  GLenum error_code = glGetError();
  if (error_code) {
    OpenGLError error("ShaderScreen.cpp", "ShaderScreen()");
    error.Code(error_code);
    error.Add("Thrown while finding attributes and uniforms in shader.");
    throw(error);
  }
  // prep attributes for buffer reading
  glBindVertexArray(GraphicPool::Screen().VertexArray());
  glBindBuffer(GL_ARRAY_BUFFER, GraphicPool::Screen().VertexBuffer());
  glVertexAttribPointer(_position_A, 2, GL_FLOAT,
                        GL_FALSE, 4 * sizeof(GLfloat), 0);
  glEnableVertexAttribArray(_position_A);
  glVertexAttribPointer(_texCoord_A, 2, GL_FLOAT, GL_FALSE,
                        4 * sizeof(GLfloat), (GLvoid*)(2 * sizeof(GLfloat)));
  glEnableVertexAttribArray(_texCoord_A);
  glBindVertexArray(0);
  glBindBuffer(GL_ARRAY_BUFFER, 0);
  // error check
  error_code = glGetError();
  if (error_code) {
    OpenGLError error("ShaderScreen.cpp", "ShaderScreen()");
    error.Code(error_code);
    error.Add("Thrown while preparing the attributes.");
    throw(error);
  }
}


/*!
\brief Gets the ID for the screen shader's position attribute.
\return The ID for the screen shader's position attribute.
*/
GLint ShaderScreen::PostionA()
{
  return _position_A;
}

/*!
\brief Gets the ID for the screen shader's texCoord attribute.
\return The ID for the screen shader's texCoord attribute.
*/
GLint ShaderScreen::TexCoordA()
{
  return _texCoord_A;
}

/*!
\brief Gets the ID for the screen shader's screen texture uniform.
\return The ID for the screen shader's screen texture uniform.
*/
GLint ShaderScreen::ScreenU()
{
  return _screen_U;
}