/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file ShaderTexture.cpp
\author Connor Deakin
\par E-mail: connor.deakin\@digipen.edu
\par Project: Graphics
\date 08/03/2017
\brief
  Implementation of Shader types that use Texture sampling.
*/
/*****************************************************************************/

#include "OpenGLError.h"

#include "ShaderTexture.h"

/*****************************************************************************/
/*!
\brief
  Creates the ShaderTexture, finds the uniforms and attributes within
  the shader, and prepares the attributes so they can read the TextureGraphic
  vertex buffer.

\param vert_file
  The shader's vertex file.
\param frag_file
  The shader's fragment file.
\param texture_graphic
  The TextureGraphic that will be displayed with this shader.
*/
/*****************************************************************************/
ShaderTexture::ShaderTexture(
  const std::string & vert_file, const std::string & frag_file,
  const TextureGraphic & texture_graphic) :
Shader(vert_file, frag_file)
{
  // getting uniform and attribute locations
  Use();
  _position_A = glGetAttribLocation(ID(), "position");
  _texCoord_A = glGetAttribLocation(ID(), "in_texCoord");
  _transformation_U = glGetUniformLocation(ID(), "transformation");
  _color_U = glGetUniformLocation(ID(), "in_color");
  _additive_U = glGetUniformLocation(ID(), "additive");
  _texture_U = glGetUniformLocation(ID(), "texture");
  // make sure attributes and uniforms were found
  CHECKATTRIBUTE(_position_A, "position",
    vert_file.c_str(), frag_file.c_str());
  CHECKATTRIBUTE(_texCoord_A, "in_texCoord", 
    vert_file.c_str(),frag_file.c_str());
  CHECKUNIFORM(_transformation_U, "transformation", 
    vert_file.c_str(), frag_file.c_str());
  CHECKUNIFORM(_color_U, "in_color", 
    vert_file.c_str(), frag_file.c_str());
  CHECKUNIFORM(_additive_U, "additive",
    vert_file.c_str(), frag_file.c_str());
  CHECKUNIFORM(_texture_U, "texture", 
    vert_file.c_str(), frag_file.c_str());
  // opengl error check
  GLenum error_code = glGetError();
  if (error_code) {
    OpenGLError error("ShaderTexture.cpp", "ShaderTexture()");
    error.Code(error_code);
    error.Add("Thrown while finding attributes and uniforms in shader.");
    throw(error);
  }
  // prep shader attributes
  glBindVertexArray(texture_graphic.VertexArray());
  glBindBuffer(GL_ARRAY_BUFFER, texture_graphic.VertexBuffer());
  glVertexAttribPointer(_position_A, 3, GL_FLOAT,
                        GL_FALSE, 5 * sizeof(GLfloat), 0);
  glEnableVertexAttribArray(_position_A);
  glVertexAttribPointer(_texCoord_A, 2, GL_FLOAT, GL_FALSE,
                        5 * sizeof(GLfloat), (GLvoid*)(3 * sizeof(GLfloat)));
  glEnableVertexAttribArray(_texCoord_A);
  glBindVertexArray(0);
  glBindBuffer(GL_ARRAY_BUFFER, 0);
  // error check
  error_code = glGetError();
  if (error_code) {
    OpenGLError error("ShaderTexture.cpp", "ShaderTexture()");
    error.Code(error_code);
    error.Add("Thrown while setting up position and texcoord "
              "attribute pointers.");
    throw(error);
  }
}

/*!
\brief Gets the location of the position attribute.
\return The location of the positon attribute.
*/
GLint ShaderTexture::PositionA() const
{
  return _position_A;
}

/*!
\brief Gets the location of the texCoord attribute.
\return The location of the texCoord attribute.
*/
GLint ShaderTexture::TexCoordA() const
{
  return _texCoord_A;
}

/*!
\brief Gets the location of the transformation uniform.
\return The location of the transformation uniform.
*/
GLint ShaderTexture::TransformationU() const
{
  return _transformation_U;
}

/*!
\brief Gets the location of the color uniform.
\return The location of the color uniform.
*/
GLint ShaderTexture::ColorU() const
{
  return _color_U;
}

/*!
\brief Gets the location of the additive uniform.
\return The location of the additive uniform.
*/
GLint ShaderTexture::AdditiveU() const
{
  return _additive_U;
}

/*!
\brief Gets the location of the texture uniform.
\return The location of the texture uniform.
*/
GLint ShaderTexture::TextureU() const
{
  return _texture_U;
}

/*****************************************************************************/
/*!
\brief
  Creates a ShaderTextureSheet and finds the uniforms that should be in the
  ShaderTextureSheet.

\param texture_graphic
  The TextureGraphic that this shader will use.
*/
/*****************************************************************************/
ShaderTextureSheet::ShaderTextureSheet(const TextureGraphic & texture_graphic) :
  ShaderTexture("Shader/TextureSheet.vert", "Shader/TextureGraphic.frag", 
                texture_graphic)
{
  // finding uniforms
  _totalFrames_U = glGetUniformLocation(ID(), "totalFrames");
  _currentFrame_U = glGetUniformLocation(ID(), "currentFrame");
  // checking that uniforms were found
  CHECKUNIFORM(_totalFrames_U, "totalFrames",
               "Shader/TextureSheet.vert", "Shader/TextureSheet.frag");
  CHECKUNIFORM(_currentFrame_U, "currentFrame", 
               "Shader/TextureSheet.vert", "Shader/TextureSheet.frag");
  // error check
  GLenum error_code = glGetError();
  if (error_code) {
    OpenGLError error("ShaderTexture.cpp", "ShaderTexture()");
    error.Code(error_code);
    error.Add("Thrown while finding attributes and uniforms in shader.");
    throw(error);
  }
}

/*!
\brief Gets the location of the totalFrames uniform.
\return The location of the totalFrames uniform.
*/
GLint ShaderTextureSheet::TotalFramesU() const
{
  return _totalFrames_U;
}

/*!
\brief Gets the location of the currentFrame uniform.
\return The location of the currentFrame uniform.
*/
GLint ShaderTextureSheet::CurrentFrameU() const
{
  return _currentFrame_U;
}