/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file Space.cpp
\author Connor Deakin
\par E-mail: connor.deakin@digipen.edu
\par Project: X-Craft
\date 20/11/2016
\brief
  Contains the implementation of the Space class.

\par
    Functions Include:
    - Update
    - Change
    - Restart
    - EndCurrent
*/
/*****************************************************************************/
#include "RendererPool.h"
#include "StateFactory.h"
#include "State.h"
#include "Context.h"
#include "Level.h"

#include "Space.h"

// pre-declarations
class State;
class Camera;

/*****************************************************************************/
/*!
\brief
  A constructor for a Space.

\param world_height
  The number of world units that should be displayed from the bottom to the top
  of the game screen.
*/
/*****************************************************************************/
Space::Space() : 
_currentState(NULL), _nextState(NULL), _camera(), _renderer(), _paused(false)
{
  RendererPool::Add(&_renderer, &_camera);
}

/*****************************************************************************/
/*!
\brief
  Constructor for a Space.

\param state
  The state that will be immediatly loaded into this Space.
\param world_height
  The number of world units displayed across the world's height.
*/
/*****************************************************************************/
Space::Space(StateType state) : 
_nextState(NULL), _camera(), _renderer(), _paused(false), _space(spaceID())
{
  RendererPool::Add(&_renderer, &_camera);
  // creating and loading the State
  State * new_state = StateFactory::Create(state);
  _currentState = new_state;
}

/*****************************************************************************/
/*!
\brief
  Creates a space that will be used to run a specific level.

\param levelFile
  The level File that will be used to create the level within this Space.
*/
/*****************************************************************************/
Space::Space(std::string levelFile) : 
  _nextState(NULL), _camera(), _renderer(), _paused(false), _space(spaceID())
{
  RendererPool::Add(&_renderer, &_camera);
  // creating and loading the State
  State * new_state = StateFactory::Create(levelFile);
  _currentState = new_state;
}

/*****************************************************************************/
/*!
\brief
  Creates a transition space that will be used to transition from one state
  to another.

\param transition_info
  Information about the transition.
*/
/*****************************************************************************/
Space::Space(const TransitionInfo & transition_info) :
  _nextState(NULL), _camera(), _renderer(), _paused(false), _space(spaceID())
{
  RendererPool::Add(&_renderer, &_camera);
  State * new_state = StateFactory::Create(transition_info);
  _currentState = new_state;
}

/*****************************************************************************/
/*!
\brief
  Copy constructor for a Space. This will copy all of the members of one Space
  into this Space

\param other
  The space that is being copied into this Space.
*/
/*****************************************************************************/
//Space::Space(const Space & other) :
//_space(other._space),
//_currentState(other._currentState), _nextState(other._nextState),
//_camera(other._camera), _worldHeight(other._worldHeight), _paused(other._paused)
//{}

State * Space::GetCurrentState()
{
  return _currentState;
}

void Space::Init()
{
  _currentState->Allocate();
  _currentState->Load();
}

/*****************************************************************************/
/*!
\brief
  Destructor for a Space. This will simply unload and free the Space's state and
  delete any dynmically allocated states.
*/
/*****************************************************************************/
Space::~Space()
{
  if(_currentState)
    EndCurrent();
  if(_nextState)
    delete _nextState;
  RendererPool::Remove(&_renderer, &_camera);
}

/*****************************************************************************/
/*!
\brief
  Upates the Space. This will manage any changes in the states display
  properties and it will Update the State / switch to a new specified state.

\return
  During the update of the State within the Space, a bool will be returned
  indicating if the game should still be running. This bool will be returned
  through the Space as well.
*/
/*****************************************************************************/
bool Space::Update()
{
  // state switch
  if (_nextState)
  {
    if (_currentState)
      EndCurrent();
    _currentState = _nextState;
    _nextState = NULL;
    _currentState->SetSpace(_space);
    _currentState->Allocate();
    _currentState->Load();
    // Fixes some physics bugs from long load times
  }

  // state machine update
  if(!_paused)
  {
    // standard update
    bool result;
    if (_currentState)
    {
      // Constructs an event to tell objects that the 
      // frame has started, and dispatches it
      DispatchEvent(&Event("FrameStart"));
      result = _currentState->Update();
      _currentState->BaseUpdate();
    }
    if (_quit)
      result = false;
    return result;
  }
  return true;
}

/*****************************************************************************/
/*!
\brief
  Used to change the space into a different State/level. A space can only
  run one state at a time.

\param state
  The new state that will be displayed and updated within this Space.
*/
/*****************************************************************************/
void Space::Change(StateType state)
{
  _nextState = StateFactory::Create(state);
}

/*****************************************************************************/
/*!
\brief
  This will restart a State by simply unloading and loading the current state.
  If you are creating objects in your States load function, they will be
  re-created during this function call.
*/
/*****************************************************************************/
void Space::Restart()
{
  if(_currentState)
  {
    _currentState->Free();
    _currentState->Unload();
    _currentState->Load();
    _currentState->Allocate();
  }
}

/*****************************************************************************/
/*!
\brief
  Call this to Pause the current Space. This means that when the Update function
  is called, the State within the Space will not be updated.
*/
/*****************************************************************************/
void Space::Pause()
{
  _paused = true;
}

/*****************************************************************************/
/*!
\brief
  Tells the space to quit at the end of this frame.
*/
/*****************************************************************************/
void Space::Quit()
{
  _quit = true;
}

/*****************************************************************************/
/*!
\brief
  Call to resume the Space so that when the Update for the Space is called, the 
  State within the Space will actually update.
*/
/*****************************************************************************/
void Space::Resume()
{
  _paused = false;
}

/*****************************************************************************/
/*!
\brief
  Get the camera that is being used to transform this Space to the correct
  camera coordinates.

\return A pointer to the space's camera.
*/
/*****************************************************************************/
Camera * Space::GetCamera(void)
{
  return &_camera;
}

Renderer & Space::GetRenderer()
{
  return _renderer;
}

/*****************************************************************************/
/*!
\brief
  Will transform SDL pixel coordinates into the Space's world coordinates.

\param pixel
  The pixel coordinate that will be changed into a world coordinate for this
  Space.
*/
/*****************************************************************************/
void Space::ScreenToWorld(glm::vec2 & pixel)
{
  Context::ScreenToAspect(pixel);
  glm::vec4 camera_coord = glm::vec4(pixel.x, pixel.y, 0.0f, 1.0f);
  camera_coord = _camera.CameraToWorld() * camera_coord;
  pixel.x = camera_coord.x;
  pixel.y = camera_coord.y;
}

/*****************************************************************************/
/*!
\brief
  This Space will be stored in a list that is managed by the SpaceManager. This
  will allow this Space to keep track of where it is stored in that list.

\param space
  The ID of the Space within the SpaceManager.s
*/
/*****************************************************************************/
void Space::SetID(spaceID space)
{
  _space = space;
  if(_currentState)
    _currentState->SetSpace(_space);
}

/*****************************************************************************/
/*!
\brief
  This will function will tell you whether this space is paused or not.

\return If the space is paused, true. If the space is not paused, false.
*/
/*****************************************************************************/
bool Space::Paused()
{
  return _paused;
}

/*****************************************************************************/
/*!
\brief
  Completely ends the current State by Freeing everything within the State and
  finally deleting the dynamically allocated State.
*/
/*****************************************************************************/
void Space::EndCurrent()
{
  _currentState->Unload();
  _currentState->Free();
  _currentState->BaseFree();
  _renderer.Clear();
  delete _currentState;
}

/*****************************************************************************/
/*!
\brief
  Get the SpaceID that is associated with this Space. This ID can be used to
  modify the Space and manipulate the Space using the SpaceManager.

\return The SpaceID for this instantiation of the state.
*/
/*****************************************************************************/
spaceID Space::GetSpaceID()
{
  return _space;
}

/*****************************************************************************/
/*!
\brief Get's the StateType that is running within this Space.

\return The StateType running in this Space.
*/
/*****************************************************************************/
StateType Space::GetStateType()
{
  return _currentState->Type();
}

/*****************************************************************************/
/*!
\brief
Adds a callback function to the list of callbacks. If there are no other
functions subscribed to that eventID, it adds that eventID.

\param eventID
This is the string ID of the event that the callback function will respond to.

\param callbackFunction
This is the callback function that will handle the event.
*/
/*****************************************************************************/
void Space::SubscribeCallback(std::string eventID, eventFunction callbackFunction, Object * object)
{
  auto itr = _callbacks.find(eventID);
  if (itr != _callbacks.end())
  {
    (*itr).second.push_back(std::pair<eventFunction, Object *>(callbackFunction, object));
  }
  else
  {
    _callbacks[eventID] = std::list<std::pair<eventFunction, Object *>>();
    _callbacks[eventID].push_back(std::pair<eventFunction, Object *>(callbackFunction, object));
  }
}

/*****************************************************************************/
/*!
\brief
Removes a callback function from the list of callbacks.

\param eventID
The eventID the function was subscribed to. This is important because the
functions are sorted by eventID, so having the ID helps me search faster.

\param callbackFunction
The function you want to remove from the list.
*/
/*****************************************************************************/
void Space::UnsubscribeCallback(std::string eventID, Object * object)
{
  auto itr = _callbacks.find(eventID);
  if (itr != _callbacks.end())
  {
    auto itr2 = (*itr).second.begin();
    for (; itr2 != (*itr).second.end(); itr2++)
    {
      if ((*itr2).second == object)
      {
        (*itr).second.erase(itr2);
        return;
      }
    }
  }
}

/*****************************************************************************/
/*!
\brief
Calls all callback functions subscribed to the event ID of the event passed in
on this space.

\param _event
Any event object. Note that it will be cast down to base class pointer, so
the callback function will have to cast back if there is more data than eventID.
Please make sure that the event you are passing has the correct eventID.
*/
/*****************************************************************************/
void Space::DispatchEvent(Event * _event)
{
  auto itr = _callbacks.find(_event->_eventID);
  if (itr != _callbacks.end())
  {
    for each (std::pair<eventFunction, Object *> callback in (*itr).second)
    {
      callback.first(_event);
    }
  }
}