/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file Space.h
\author Connor Deakin
\par E-mail: connor.deakin@digipen.edu
\par Project: Mecha
\date 20/11/2016
\brief
  Contains the interface for the Space class. Look at the Space class for more
  information.
*/
/*****************************************************************************/
#ifndef SPACE_H
#define SPACE_H

#include <map>

#include "StateFactory.h"
#include "spaceID.h"
#include "Event.h"
#include "Camera.h"
#include "Renderer.h"


// pre-declarations
class Camera;
class State;


/*****************************************************************************/
/*!
\class Space
\brief
  A space defines a place for a State (level) to be displayed. To do this, a
  Space consists of three primary elements.
  - Current State
  - World Height
  - Level Camera
  These are described in more depth in the member variable comments. With
  these three properties, a Space has the ability to run a level and display it
  properly to the screen. This also means one level can be displayed over
  another.

\par
    Operations include:
    - Creating a Space
    - Updating a Space
    - Changing the State the Space displays
    - Restarting the State displayed by the Space.

\deprecated
*/
/*****************************************************************************/
class Space
{
  public:
    Space();
    Space(StateType state);
    Space(std::string levelFile);
    Space(const TransitionInfo & transition_info);
    //Space(const Space & other);
    ~Space();
    void Init();
    bool Update();
    void Change(StateType state);
    void Restart();
    void Pause();
    void Resume();
    Camera * GetCamera(void);
    Renderer & GetRenderer();
    void ScreenToWorld(glm::vec2 & pixel);
    void SetID(spaceID space);
    bool Paused();
    StateType GetStateType();
    spaceID GetSpaceID();
    State * GetCurrentState();
    void Quit();


    void SubscribeCallback(std::string eventID,  eventFunction callbackFunction, Object * object);
    // Note: The reason you need to pass in an eventID is because the functions are sorted by eventID,
    // so passing in an eventID speeds up the search.
    void UnsubscribeCallback(std::string eventID, Object * object);

    void DispatchEvent(Event * _event);

  private:
    void EndCurrent();
    //! Since all Spaces are created by the SpaceManager, we need an ID to
    // reference where this Space is within the SpaceManger. This is what this
    // ID is here for.
    spaceID _space;
    //! This is a pointer to the current State that the Space is running on
    // its update function.
    State * _currentState;
    //! When a state change request is made, this will store a pointer
    // to the new state that is to be used until the Space is updated again.
    State * _nextState;
    //! The camera used in this Space.
    Camera _camera;
    //! The Renderer used to display sprites in this Space.
    Renderer _renderer;
    //! The number of units in world units to display across the game window's
    // height.
    float _worldHeight;
    //! Tracks whether this Space is paused or not. If the space is paused, it
    // will not update when the Space's update is called.
    bool _paused;
    //! If this is set, the space will return false regardless of the return value of the state update.
    bool _quit;
    std::map<std::string, std::list<std::pair<eventFunction, Object *> > > _callbacks;
};

#endif // !SPACE_H

