/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file SpaceManager.cpp
\author Connor Deakin
\par E-mail: connor.deakin@digipen.edu
\par Project: X-Craft
\date 29/11/2016
\brief
  Contains the implementation of the SpaceManager.

\par
    Functions Include:
    - Add
    - Remove
    - ResumeAll
*/
/*****************************************************************************/

#include <vector>

#include "Error.h"
#include "AudioEngine.h"
#include "TexturePool.h"
#include "CollisionManager.h"
#include "RendererPool.h"

#include "SpaceManager.h"

//pre-declarations
class Space;

namespace Global
{

// static initializations
std::queue<spaceID> SpaceManager::_deadSpaces = std::queue<spaceID>();
std::list<Space *> SpaceManager::_spaces = std::list<::Space *>();

/*****************************************************************************/
/*!
\brief
  Use this to add a new Space the game's SpaceManager. When the new Space is
  added, the Space will begin displaying and updating when the SpaceManager is
  updated.

\param state
  The type of state that will be displayed with the new Space.
\param world_height
  The height displayed on the game window in world units.

\return An spaceID that can be used to remove the Space from the SpaceManager
  later on or manipulate the Space.
*/
/*****************************************************************************/
spaceID SpaceManager::Add(StateType state, float world_height)
{
  _spaces.push_back(new ::Space(state));
  // return iterator (spaceID) to last element
  spaceID space = --_spaces.end();
  (**space).GetCamera()->SetHeight(world_height);
  (**space).SetID(space);
  (**space).Init();
  return space;
}

/*****************************************************************************/
/*!
\brief
  Add a level to the space manager.

\param fileName
  The name of the level file.
\param world_height
  The height of the world on the screen.

\return A space ID for the space created.
*/
/*****************************************************************************/
spaceID SpaceManager::Add(std::string fileName, float world_height)
{
  _spaces.push_back(new ::Space(fileName));
  // return iterator (spaceID) to last element
  spaceID space = --_spaces.end();
  (**space).GetCamera()->SetHeight(world_height);
  (**space).SetID(space);
  (**space).Init();
  return space;
}

/*****************************************************************************/
/*!
\brief
  Adds a space to the SpaceManager that will create a smooth transtion from
  one space to another.

\param transition_info
  Information about the transition that is going to take place.
*/
/*****************************************************************************/
spaceID SpaceManager::Add(const TransitionInfo & transition_info)
{
  _spaces.push_back(new ::Space(transition_info));
  // return iterator (spaceID) to last element
  spaceID space = --_spaces.end();
  (**space).GetCamera()->SetHeight(1.0f);
  (**space).SetID(space);
  (**space).Init();
  return space;
}

/*****************************************************************************/
/*!
\brief
  Adds a space above the given space.

\param space
  The space that the new space will be above.
\param state
  The state of the new space.
\param world_height
  The world height of the new space.

\return An ID for the new space created.
*/
/*****************************************************************************/
spaceID SpaceManager::AddAbove(spaceID space, StateType state, 
  float world_height)
{
  _spaces.insert(++space, new ::Space(state));
  // setup the new space
  spaceID new_space = --space;
  (**new_space).GetCamera()->SetHeight(world_height);
  (**new_space).SetID(new_space);
  (**new_space).Init();
  ReorderRenderers();
  return new_space;
}

/*****************************************************************************/
/*!
\brief
  Adds a space to the bottom of the SpaceManager.

\param state
  The state that will be used in the space.
\param world_height
  The initial height of the world when the space is displayed.
*/
/*****************************************************************************/
spaceID SpaceManager::AddBottom(StateType state, float world_height)
{
  _spaces.push_front(new ::Space(state));
  // set up space
  spaceID new_space = _spaces.begin();
  (**new_space).GetCamera()->SetHeight(world_height);
  (**new_space).SetID(new_space);
  (**new_space).Init();
  ReorderRenderers();
  return new_space;
}

/*****************************************************************************/
/*!
\brief
  Adds a level to the bottom of the SpaceManager.

\param filename
  The level file that will will be used for the new space.
\param world_height
  The initial height of the world when the space is displayed.
*/
/*****************************************************************************/
spaceID SpaceManager::AddBottom(const std::string & filename,
  float world_height)
{
  _spaces.push_front(new ::Space(filename));
  // set up space
  spaceID new_space = _spaces.begin();
  (**new_space).GetCamera()->SetHeight(world_height);
  (**new_space).SetID(new_space);
  (**new_space).Init();
  ReorderRenderers();
  return new_space;
}

/*****************************************************************************/
/*!
\brief
  Used to remove a Space currently in the SpaceManager from the SpaceManager.
  The Space will not actually be removed from the SpaceManager until the next
  time the SpaceManager's update function is called.

\param space
  An ID to the Space that needs to be removed. This ID was givn by the
  SpaceManager's add function.
*/
/*****************************************************************************/
void SpaceManager::Remove(spaceID space)
{
  std::list<::Space *>::iterator it = _spaces.begin();
  std::list<::Space *>::iterator it_e = _spaces.end();
  for (; it != it_e; ++it) {
    if (it == space) {
      _deadSpaces.push(space);
      StateFactory::DeActivate((**space).GetStateType());
      return;
    }
  }
}

/*****************************************************************************/
/*!
\brief
  Sets all the Spaces that are currently in the SpaceManager to be destroyed
  during the next time the SpaceManager is updated. A Space can be added to
  the SpaceManager directly after calling this function, and that added Space
  will not be removed from the SpaceManager.
*/
/*****************************************************************************/
void SpaceManager::RemoveAll()
{
  std::list<::Space *>::iterator it = _spaces.begin();
  std::list<::Space *>::iterator it_e = _spaces.end();
  for (; it != it_e; ++it) {
    _deadSpaces.push(it);
    StateFactory::DeActivate((*it)->GetStateType()); // HACK
  }
}

/*****************************************************************************/
/*!
\brief
  Removes all of the spaces that are beneath a given space.

\param space_index
  The first space that won't be removed from the SpaceManager. All Spaces below
  this will be removed.
*/
/*****************************************************************************/
void SpaceManager::RemoveAllBeneath(spaceID space_index)
{
  // removing all beneath spaces
  std::list<::Space *>::iterator it = _spaces.begin();
  std::list<::Space *>::iterator it_e = _spaces.end();
  for (; it != it_e && it != space_index; ++it) {
    _deadSpaces.push(it);
    StateFactory::DeActivate((*it)->GetStateType()); // HACK
  }
  if (it != space_index) {
    // space_index not in SpaceManager.
    Error error("SpaceManager.cpp", "RemoveAllBeneath");
    error.Add("The spaceID given to this function did not exist in the "
      "SpaceManager");
    throw(error);
  }
}

/*****************************************************************************/
/*!
\brief
  Updates every Space within the SpaceManager, after removing the dead Spaces
  within the SpaceManager.

\return
  The SpaceManager will Update all of the Spaces. If any of the Spaces returns
  false during the Update, this function will return false, indicating that the
  game should be shut down.
*/
/*****************************************************************************/
bool SpaceManager::Update()
{
  RemoveDeadSpaces();
  // update all spaces
  std::list<::Space *>::iterator it = _spaces.begin();
  std::list<::Space *>::iterator it_e = _spaces.end();
  for (it; it != it_e; ++it)
  {
    // In the future, each space will be updating this
    bool result;
    result = (*it)->Update();
    if(!result)
      return false;
  }
  return true;
}

/*****************************************************************************/
/*!
\brief
  This will Pause all the Spaces that are currently in the SpaceManager. Even
  if the Space was added the line before the call to Pause all the Spaces in
  the SpaceManager, the Space will be paused.
*/
/*****************************************************************************/
void SpaceManager::PauseAll()
{
  std::list<::Space *>::iterator it = _spaces.begin();
  std::list<::Space *>::iterator it_e = _spaces.end();
  for (it; it != it_e; ++it)
    (**it).Pause();
}

/*****************************************************************************/
/*!
\brief
  Will resume all Spaces in the SpaceManager. If a Space within the manager is
  not paused, this function will have no effect on that specific space.
*/
/*****************************************************************************/
void SpaceManager::ResumeAll()
{
  // resume all spaces
  std::list<::Space *>::iterator it = _spaces.begin();
  std::list<::Space *>::iterator it_e = _spaces.end();
  for (it; it != it_e; ++it)
    (**it).Resume();
}


/*****************************************************************************/
/*!
\brief
  Will restart all Spaces in the SpaceManager.
*/
/*****************************************************************************/
void SpaceManager::RestartAll()
{
  audEngine.SetEventParameter(audEngine.GetMusicEvent("LevelOne"),
                              "Pause", 0);
  // restart all spaces
  std::list<::Space *>::iterator it = _spaces.begin();
  std::list<::Space *>::iterator it_e = _spaces.end();
  for (it; it != it_e; ++it)
    (**it).Restart();
}

/*****************************************************************************/
/*!
\brief
  This function will Resume the Space that is beneath the Space given. For
  exaple, if a pause menu Space was above a game Space, this function would
  resume the game Space given the ID of the pause menu Space. If the Space is
  not paused, this will have no effect.

\par Important Notes
  - If you attempt to resume the space beneath the only existing or the first
    space, an exception will be thrown.

\param space
  The ID for the Space that is above the Space to be resumed.
*/
/*****************************************************************************/
void SpaceManager::ResumeBeneath(spaceID space)
{
  if(space != _spaces.begin())
  {
    --space;
    (**space).Resume();
  }
  else
  {
    Error error("SpaceManager.cpp", "ResumeBeneath");
    error.Add("You are attempting to resume the space under the bottom space.");
    error.Add("This space does not exist.");
  }
}

/*****************************************************************************/
/*!
\brief
  This will restart the Space that is beneath the Space given. For
  exaple, if a pause menu Space was above a game Space, this function would
  restart the game Space given the ID of the pause menu Space.

\param
  The ID for the Space that is above the Space that will be restarted.
*/
/*****************************************************************************/
void SpaceManager::RestartBeneath(spaceID space)
{
  --space;
  if (space != (--_spaces.begin()))
    (**space).Restart();
}


/*!
\brief Get the Space that is underneath a given Space.
\param space The given space.
*/
spaceID SpaceManager::SpaceUnderneath(spaceID space)
{
  if (space == _spaces.begin()) {
    Error error("SpaceManager.cpp", "SpaceUnderneath");
    error.Add("There is no Space underneath the first Space.");
    throw(error);
  }
  return (--space);
}

/*****************************************************************************/
/*!
\brief
  This will search through every Space within the SpaceManager to find a
  State that is running a specific StateType. For example, if you wanted to
  find the State running MENU_PAUSE StateType, then all you need to do is give
  this function the MENU_PAUSE StateType and it will find that State for you.

\par Important Notes
  - If the StateType is not found, NULL is returned.

\param state
  The StateType you are searching for.

\return A pointer to the State that is running that StateType.
*/
/*****************************************************************************/
State * SpaceManager::FindState(StateType state)
{
  std::list<::Space *>::iterator it = _spaces.begin();
  std::list<::Space *>::iterator it_e = _spaces.end();
  for (it; it != it_e; ++it)
  {
    if((*it)->GetStateType() == state)
      return (*it)->GetCurrentState();
  }
  return NULL;
}
/*****************************************************************************/
/*!
\brief
  Destroys all of the resources within the SpaceManager.
*/
/*****************************************************************************/
void SpaceManager::Purge()
{
  // delete spaces
  std::list<::Space *>::iterator it = _spaces.begin();
  std::list<::Space *>::iterator it_e = _spaces.end();
  for (it; it != it_e; ++it)
    delete *it;
  // clear the list
  _spaces.clear();
}

/*****************************************************************************/
/*!
\brief
  This is called during the SpaceManager's update. This will remove all of the
  Space's that are in the queue that was populated by the SpaceManager::Remove
  function. Once this is called. All of the dead spaces will no longer be in
  the SpaceManager.
*/
/*****************************************************************************/
void SpaceManager::RemoveDeadSpaces()
{
  while(!_deadSpaces.empty())
  {
    if (*_deadSpaces.front()) {
      delete (*_deadSpaces.front());
      _spaces.erase(_deadSpaces.front());
      _deadSpaces.pop();
    }
  }
}

/*!
\brief Reorders the renders so that they display in the order of the Spaces.
*/
void SpaceManager::ReorderRenderers()
{
  RendererPool::Clear();
  std::list<::Space *>::iterator it = _spaces.begin();
  std::list<::Space *>::iterator it_e = _spaces.end();
  for (; it != it_e; ++it)
    RendererPool::Add(&(**it).GetRenderer(), (**it).GetCamera());
}

/*****************************************************************************/
/*!
\brief
  Returns the value of a null spaceID. This is actually the end of the list iterator.

\return
  Returns the current null spaceID. Note that this will be invalidated when the list is updated.
*/
/*****************************************************************************/
spaceID SpaceManager::NullSpaceID()
{
  return _spaces.end();
}

} // Global
