/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file SpritePool.cpp
\author Connor Deakin
\par E-mail: connor.deakin\@digipen.edu
\par Project: Graphics
\date 16/03/2017
\brief

*/
/*****************************************************************************/

#include "SpritePool.h"

//! GraphicSprite pointer vector.
typedef std::vector<GraphicSprite *> vgs;
//! TextureSprite pointer vector.
typedef std::vector<TextureSprite *> vts;
//! TextureSheetSprite pointer vector.
typedef std::vector<TextureSheetSprite *> vtss;

// static initializations
vgs SpritePool::_graphicSprites = vgs();
vts SpritePool::_textureSprites = vts();
vtss SpritePool::_textureSheetSprites = vtss();

/*****************************************************************************/
/*!
\brief
  Add a GraphicSprite to SpritePool.

\param shader
  The shader that will be used to display the GraphicSprite.

\return The new GraphicSprite.
*/
/*****************************************************************************/
GraphicSprite * SpritePool::AddGraphic(const GraphicShader * shader)
{
  GraphicSprite * new_sprite = new GraphicSprite(shader);
  _graphicSprites.push_back(new_sprite);
  return new_sprite;
}

/*****************************************************************************/
/*!
\brief
  Adds a TextureSprite to the SpritePool.

\param shader
  The shader used to display the TextureSprite.
\param texture
  The texture displayed on the TextureSprite.

\return The new TextureSprite added to the SpritePool.
*/
/*****************************************************************************/
TextureSprite * SpritePool::AddTexture(const ShaderTexture * shader,
                                      const Texture * texture)
{
  TextureSprite * new_sprite = new TextureSprite(shader, texture);
  _textureSprites.push_back(new_sprite);
  return new_sprite;
}

/*****************************************************************************/
/*!
\brief
  Adds a TextureSheetSprite to the SpritePool.

\param shader
  The shader used to when drawing the TextureSheetSprite.
\param texture_sheet
  The TextureSheet displayed on the TextureSheetSprite.

\return The new TextureSheetSprite.
*/
/*****************************************************************************/
TextureSheetSprite * 
SpritePool::AddTextureSheet(const ShaderTextureSheet * shader,
                            const TextureSheet * texture_sheet)
{
  TextureSheetSprite * new_sprite = new TextureSheetSprite(shader, 
                                                           texture_sheet);
  _textureSheetSprites.push_back(new_sprite);
  return new_sprite;
}

//! Dealocate all memory in an iterator range.
#define DELETE()\
for(;it != it_e; ++it)\
  delete *it;

/*****************************************************************************/
/*!
\brief
  Deallocates all of the Sprites in the SpritePool.
*/
/*****************************************************************************/
void SpritePool::Purge()
{
  { vgs::iterator it = _graphicSprites.begin();
    vgs::iterator it_e = _graphicSprites.end();
    DELETE(); }
  { vts::iterator it = _textureSprites.begin();
    vts::iterator it_e = _textureSprites.end();
    DELETE(); }
  { vtss::iterator it = _textureSheetSprites.begin();
    vtss::iterator it_e = _textureSheetSprites.end();
    DELETE(); }

}