/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file GameStatec.cpp
\author Connor Deakin
\par E-mail: connor.deakin@digipen.edu
\par Project: X-Craft
\date 26/11/2016
\brief
  Contains the minimal implementation of the abstract State class.

\par
    Functions Include:
    - State
*/
/*****************************************************************************/
#include "InputManager.h"
#include "GameObjectManager.h"
#include "SpaceManager.h"
#include "StateFactory.h"

#include "State.h"

/*****************************************************************************/
/*!
\brief
  Default constructor for the State class. This will simply create an empty
  GameObjectManager to be used by the derived state.
*/
/*****************************************************************************/
State::State() : _objectManager(GameObjectManager())
{}

GameObjectManager * State::GetObjectManager()
{
  return &_objectManager;
}

/*****************************************************************************/
/*!
\brief
  Used to set the Space the this state is stored within. This will allow one
  to manipulate the containing Space from within the State itself.

\param space
  An ID to the Space that this State is contained in.
*/
/*****************************************************************************/
void State::SetSpace(spaceID space)
{
  _space = space;
  _objectManager.SetSpaceID(_space);
}

/*****************************************************************************/
/*!
\brief
  Sets which StateType exists within this State. This only to be used by
  the state factory.

\param state
  The StateType of this State.
*/
/*****************************************************************************/
void State::SetType(StateType state)
{
  _type = state;
}

/*****************************************************************************/
/*!
\brief Returns the type of State that exists within the State.

\return The StateType that is being ran in this State.
*/
/*****************************************************************************/
StateType State::Type(void)
{
  return _type;
}

/*****************************************************************************/
/*!
\brief
  Empty Init function that can be overloaded. Used for adding resources that
  will not changed if the level is reloaded.
*/
/*****************************************************************************/
void State::Allocate(){}
/*****************************************************************************/
/*!
\brief
  Empty Init function that can be overloaded. Used to load the level using
  current resources.
*/
/*****************************************************************************/
void State::Load(){}
/*****************************************************************************/
/*!
\brief
  Empty Update function that can be overloaded. Updates the State.
*/
/*****************************************************************************/
bool State::Update(){ return true; }
/*****************************************************************************/
/*!
\brief
  Empty Unload function that can be overloaded. Unloads the State.
*/
/*****************************************************************************/
void State::Unload(){}
/*****************************************************************************/
/*!
\brief
  Empty Free function that can be overloaded. Frees all the initialized data
  from the state.
*/
/*****************************************************************************/
void State::Free(){}

/*****************************************************************************/
/*!
\brief
  This will be used to perform any updates that are related to every level. For
  example, every State will need to have its objectManager updated every frame.
*/
/*****************************************************************************/
void State::BaseUpdate()
{
  _objectManager.Update();
}

/*****************************************************************************/
/*!
\brief
  This will be used to free any data that is related to every level. For
  example, every state will have a ObjectManager that must be cleared. This
  will also deactive the state type so the type of state can be creatd again.
*/
/*****************************************************************************/
void State::BaseFree()
{
  _objectManager.ClearObjects();
  StateFactory::DeActivate(_type);
}

const std::vector<TileMap *> & State::GetTileMaps()
{
  return _tileMaps;
}