/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file StateFactory.cpp
\author Connor Deakin
\par E-mail: connor.deakin@digipen.edu
\par Project: X-Craft
\date 29/11/2016
\brief
  Contains the implementation of the StateFactory. A class used to create new
  States that are managed by Spaces.

\par
    Functions Include:
    - StateFactory::CreateState
*/
/*****************************************************************************/
#include "States.h"
#include "Error.h"

#include "StateFactory.h"

// Static Initializations /////////////////////////////////////////////////////
bool StateFactory::_active[NUM_STATES] = { false };

/*****************************************************************************/
/*!
\brief
  Used to create a new State. This will dynamically allocated a State so that it
  can be easily used by a Space. If you are using this for something other than
  SpaceManagement, make sure you know what you're doing.

\param state
  The type of the State being added. These State types are defined in the
  StateType enumeration.

\return A pointer to the newly created State. You must delete this when you are
  done.
*/
/*****************************************************************************/
State * StateFactory::Create(StateType state)
{
  // make sure the state doesn't already exist
  if (_active[state])
  {
    Error error("StateFactory.cpp", "Create");
    error.Add("Multiple forms of the same StateType cannot exist.");
    throw(error);
  }
  // create new state
  State * new_state;
  switch (state)
  {
    case MENU_PAUSE:        new_state = new MenuPause;        break;
    case MENU_MAIN:         new_state = new MenuMain;         break;
    case HUD_OVERLAY:       new_state = new HUDoverlay;       break;
    case LEVEL_CHEAT:       new_state = new LevelCheat;       break;
    case LEVEL_ONE:         new_state = new Level("Resources/levels/Level1.lvl");         break;
    case SPLASH_SCREEN:     new_state = new SplashScreen;     break;
    case CREDITS_SCREEN:    new_state = new CreditsScreen;    break;
    case CONTROLS_SCREEN:   new_state = new ControlsScreen;   break;
    case SETTINGS_MENU:     new_state = new SettingsMenu;     break;
    case LOGO_SCREEN:       new_state = new LogoScreen;       break;
    case WIN_SCREEN:        new_state = new CreditsScreen;    break;
    case LOSE_SCREEN:       new_state = new LoseScreen;       break;
    case DRIVER_INVENTORY:  new_state = new DriverInventory;  break;
    case DIALOGUE_LEVEL:    new_state = new DialogueOverlay;  break;
    default:                new_state = new MenuMain;  break;
  }
  new_state->SetType(state);
  _active[state] = true;
  return new_state;
}

State * StateFactory::Create(std::string level)
{
  State * new_state =  new Level(level);
  new_state->SetType(LEVEL_ONE);
  _active[LEVEL_ONE] = true;
  return new_state;
}

/*****************************************************************************/
/*!
\brief
  Creates a transtion state which will cause the screen to fade to black and
  then fade to another state.

\param transition_info
  Information about the transition taking place. Information on the class
  can be found in the StateFactory header.

\return A pointer to the new State.
*/
/*****************************************************************************/
State * StateFactory::Create(const TransitionInfo & transition_info)
{
  State * new_state = new Transition(transition_info);
  new_state->SetType(TRANSITION);
  _active[TRANSITION] = true;
  return new_state;
}

/*****************************************************************************/
/*!
\brief
  Tells you whether a specific state has already been created.

\param state
  The type of state that you are wondering about.

\return If the state type has been created, true. If the state has not been
  created, false.
*/
/*****************************************************************************/
bool StateFactory::Active(StateType state)
{
  return _active[state];
}

/*****************************************************************************/
/*!
\brief
  Tells the state factory that a StateType is no longer being used.

\param state
  The state that is now available for use.
*/
/*****************************************************************************/
void StateFactory::DeActivate(StateType state)
{
  _active[state] = false;
}

/*****************************************************************************/
/*!
\brief
  A constructor that does absolutely nothing. It can't even be used since it's
  private.
*/
/*****************************************************************************/
StateFactory::StateFactory()
{}