/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file StateFactory.h
\author Connor Deakin
\par E-mail: connor.deakin@digipen.edu
\par Project: X-Craft
\date 29/11/2016
\brief
  Contains the interface for the StateFactory, a class used to dynamically
  create a State that can be ran within a Space.
*/
/*****************************************************************************/
#ifndef GAMESTATEFACTORY_H
#define GAMESTATEFACTORY_H

#include <string>

// pre-declarations
class State;

/*****************************************************************************/
/*!
\enum StateType
\brief
  All of these values specify the levels that the StateFactory can create.
  This enum is primarily used by the CreateState function in the StateFactory.

\par Important Notes
  - Do not set the members in this enum to specific integer values.
*/
/*****************************************************************************/
enum StateType
{
  TRANSITION,
  MENU_PAUSE,
  MENU_MAIN,
  HUD_OVERLAY,
  LEVEL_CHEAT,
  LEVEL_ONE,
  SPLASH_SCREEN,
  LOGO_SCREEN,
  CREDITS_SCREEN,
  CONTROLS_SCREEN,
  SETTINGS_MENU,
  WIN_SCREEN,
  LOSE_SCREEN,
  DRIVER_INVENTORY,
  GUNNER_INVENTORY,
  DIALOGUE_LEVEL,
  NUM_STATES
};

/*****************************************************************************/
/*!
\class TransitionInfo
\brief
  Stores the information that is used to transition from one state / level to
  another.
*/
/*****************************************************************************/
struct TransitionInfo
{
public:
  /*!
  \brief Preps a TransitionInfo instance for use.
  \param fade_time The amount of time it takes to perform a fade.
  \param rest_time The amount of time the screen remains black for.
  \param next_state The state type that is being switched to.
  \param level_file The level that is being switched to. This does not
    need to be specified if you are not switching to a level.
  */
  TransitionInfo(float fade_time, float rest_time,  StateType next_state,
    const std::string & level_file = std::string()) :
    _fadeTime(fade_time), _restTime(rest_time), _nextState(next_state),
    _levelFile(level_file) {}
  //! The time it takes to fade to black / fade from black.
  float _fadeTime;
  //! The amount of time the screen will be completely black for.
  float _restTime;
  //! The state that will be loaded in after the fade.
  StateType _nextState;
  //! The level file that should be loaded in. If there is no level that is to
  // be loaded, this does not need to be specified.
  std::string _levelFile;
};

/*****************************************************************************/
/*!
\class StateFactory
\brief
  This class is used to dynmically create States that can be used within Spaces.
  When using this, the state will be dynamically allocated and it is your job to
  delete the State when you are done. If you do not do this, it will result in
  a leak.

\par
  Important Notes:
  - This class should only be used by the Space class and the SpaceManager. If
    you are using it for something else, please make sure you really want to do
    that.
*/
/*****************************************************************************/
class StateFactory
{
  friend State;
  public:
 
    static State * Create(StateType state);
    static State * Create(std::string levelFile);
    static State * Create(const TransitionInfo & transition_info);
    static bool Active(StateType state);
    static void DeActivate(StateType state);
  private:
    //! An array of booleans that tell whether specific state types are active.
    // Each index refers to a StateType. If that StateType is active, the value
    // will be true, and vice versa.
    static bool _active[NUM_STATES];
    StateFactory();
};

#endif // GAMESTATEFACTORY_H