/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file Texture.cpp
\author Connor Deakin
\par E-mail: connor.deakin\@digipen.edu
\par Project: Graphics
\date 25/02/2017
\brief
  Contains the implementation fo the Texture class. Look in Texture.h for more
  info.
*/
/*****************************************************************************/

#include <SOIL/SOIL.h>

#include "OpenGLError.h"

#include "Texture.h" 

/*****************************************************************************/
/*!
\brief
  Given an image file, this will create the texture so it can be used to be
  drawn onto a graphic after binding.

\param image_file
  The path the the image file being used to generate the Texture.
*/
/*****************************************************************************/
Texture::Texture(const std::string & image_file) : _textureFile(image_file)
{
  // GL texture creation
  glGenTextures(1, &_texture);
  glBindTexture(GL_TEXTURE_2D, _texture);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);
  glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
  // load image from file
  unsigned char * image_data = SOIL_load_image(image_file.c_str(), &_width, 
                                               &_height, 0, SOIL_LOAD_RGBA);
  glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, _width, _height, 0, GL_RGBA, 
               GL_UNSIGNED_BYTE, image_data);
  glGenerateMipmap(GL_TEXTURE_2D);
  SOIL_free_image_data(image_data);
  glBindTexture(GL_TEXTURE_2D, 0);
  _aspectRatio = (float)_width / (float)_height;
  // error check
  GLenum error = glGetError();
  if (error) {
    OpenGLError exception("Texture", "Texture()");
    exception.Code(error);
    exception.Add("Occured while loading texture data.");
    exception.Add("- The following file did not load - ");
    exception.Add(image_file.c_str());
    throw(exception);
  }
}

/*****************************************************************************/
/*!
\brief
  This will delete that texture buffer that is created by OpenGL.
*/
/*****************************************************************************/
void Texture::Purge()
{
  glDeleteTextures(1, &_texture);
}

/*****************************************************************************/
/*!
\brief
  Binds the texture so it is drawn on a Graphic.
*/
/*****************************************************************************/
void Texture::Bind() const
{
  glBindTexture(GL_TEXTURE_2D, _texture);
}

/*****************************************************************************/
/*!
\brief
  Returns the width of the Texture in pixels.

\return The Texture width.
*/
/*****************************************************************************/
int Texture::Width() const
{
  return _width;
}

/*****************************************************************************/
/*!
\brief
  Returns the height of the Texture in pixels.

\return The Texture height.
*/
/*****************************************************************************/
int Texture::Height() const
{
  return _height;
}

/*****************************************************************************/
/*!
\brief
  The aspect ratio of the texture.

\return The texture's aspect ratio (width / height).
*/
/*****************************************************************************/
float Texture::AspectRatio() const
{
  return _aspectRatio;
}

/*****************************************************************************/
/*!
\brief
  Returns the name of the file that the Texture was loaded from.

\return The Texture's load file name.
*/
/*****************************************************************************/
const std::string & Texture::FileName() const
{
  return _textureFile;
}