/*****************************************************************************/
/*!
\file TextureGraphicShader.cpp
\author Connor Deakin
\par E-mail: connor.deakin\@digipen.edu
\par Project: Graphics
\date 08/03/2017
\brief
  Implementation of the TextureGraphicShader.
*/
/*****************************************************************************/

#include "OpenGLError.h"

#include "TextureGraphicShader.h"

/*****************************************************************************/
/*!
\brief
  Creates the TextureGraphicShader, finds the uniforms and attributes within
  the shader, and prepares the attributes so they can read the TextureGraphic
  vertex buffer.

\param vert_file
  The shader's vertex file.
\param frag_file
  The shader's fragment file.
\param texture_graphic
  The TextureGraphic that will be displayed with this shader.
*/
/*****************************************************************************/
TextureGraphicShader::TextureGraphicShader(
  const std::string & vert_file, const std::string & frag_file,
  const TextureGraphic & texture_graphic) :
Shader(vert_file, frag_file)
{
  // getting uniform and attribute locations
  Use();
  _position_A = glGetAttribLocation(ID(), "position");
  _texCoord_A = glGetAttribLocation(ID(), "in_texCoord");
  _transformation_U = glGetUniformLocation(ID(), "transformation");
  _color_U = glGetUniformLocation(ID(), "in_color");
  _texture_U = glGetUniformLocation(ID(), "texture");
  // make sure attributes and uniforms were found
  CHECKATTRIBUTE(_position_A, "position",
    vert_file.c_str(), frag_file.c_str());
  CHECKATTRIBUTE(_texCoord_A, "in_texCoord", 
    vert_file.c_str(),frag_file.c_str());
  CHECKUNIFORM(_transformation_U, "transformation", 
    vert_file.c_str(), frag_file.c_str());
  CHECKUNIFORM(_color_U, "in_color", 
    vert_file.c_str(), frag_file.c_str());
  CHECKUNIFORM(_texture_U, "texture", 
    vert_file.c_str(), frag_file.c_str());
  // opengl error check
  GLenum error_code = glGetError();
  if (error_code) {
    OpenGLError error("TextureGraphicShader.cpp", "TextureGraphicShader()");
    error.Code(error_code);
    error.Add("Thrown while finding attributes and uniforms in shader.");
    throw(error);
  }
  // prep shader attributes
  glBindVertexArray(texture_graphic.VertexArray());
  glBindBuffer(GL_ARRAY_BUFFER, texture_graphic.VertexBuffer());
  glVertexAttribPointer(_position_A, 3, GL_FLOAT,
                        GL_FALSE, 5 * sizeof(GLfloat), 0);
  glEnableVertexAttribArray(_position_A);
  glVertexAttribPointer(_texCoord_A, 2, GL_FLOAT, GL_FALSE,
                        5 * sizeof(GLfloat), (GLvoid*)(3 * sizeof(GLfloat)));
  glEnableVertexAttribArray(_texCoord_A);
  glBindVertexArray(0);
  glBindBuffer(GL_ARRAY_BUFFER, 0);
  // error check
  error_code = glGetError();
  if (error_code) {
    OpenGLError error("TextureGraphicShader.cpp", "TextureGraphicShader()");
    error.Code(error_code);
    error.Add("Thrown while setting up position and texcoord "
              "attribute pointers.");
    throw(error);
  }
}

/*!
\brief Gets the location of the position attribute.
\return The location of the positon attribute.
*/
GLint TextureGraphicShader::PositionA() const
{
  return _position_A;
}

/*!
\brief Gets the location of the texCoord attribute.
\return The location of the texCoord attribute.
*/
GLint TextureGraphicShader::TexCoordA() const
{
  return _texCoord_A;
}

/*!
\brief Gets the location of the transformation uniform.
\return The location of the transformation uniform.
*/
GLint TextureGraphicShader::TransformationU() const
{
  return _transformation_U;
}

/*!
\brief Gets the location of the color uniform.
\return The location of the color uniform.
*/
GLint TextureGraphicShader::ColorU() const
{
  return _color_U;
}

/*!
\brief Gets the location of the texture uniform.
\return The location of the texture uniform.
*/
GLint TextureGraphicShader::TextureU() const
{
  return _texture_U;
}