/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file TexturePool.cpp
\author Connor Deakin
\par E-mail: connor.deakin\@digipen.edu
\par Project: Graphics
\date 14/03/2017
\brief
  Implementation of the TexturePool.
*/
/*****************************************************************************/

#include <iostream>

#include "Error.h"

#include "TexturePool.h"

//! name texture map : ntmap
typedef std::map<std::string, Texture *> ntmap;
//! name texture sheet map : ntsmap
typedef std::map<std::string, TextureSheet *> ntsmap;

// static initializations
ntmap TexturePool::_pool = ntmap();
ntsmap TexturePool::_sheetPool = ntsmap();

/*****************************************************************************/
/*!
\brief
  Add a new Texture to the TexturePool.

\param filename
  The file in which the Texture is stored.

\return A pointer to the new Texture.
*/
/*****************************************************************************/
Texture * TexturePool::Add(const std::string & filename)
{
  // already exists in pool
  ntmap::const_iterator it;
  it = _pool.find(filename);
  if (it != _pool.end())
    return it->second;
  // allocation
  Texture * new_texture = nullptr;
  try { new_texture = new Texture(filename); }
  catch (const Error & error) { std::cout << error; }
  // add to pool
  _pool[filename] = new_texture;
  return new_texture;
}

/*****************************************************************************/
/*!
\brief
  Used to add a TextureSheet to the TexturePool

\param filename
  The file in which the TextureSheet is located.
\param total_frames
  The total number of frames in the TextureSheet.
\param rows
  The total number of rows in the TextureSheet.
\param columns
  The total number of columns in the TextureSheet.

\return A pointer to the new TextureSheet.
*/
/*****************************************************************************/
TextureSheet * TexturePool::AddSheet(const std::string & filename,
                                     unsigned int total_frames,
                                     unsigned int rows, unsigned int columns)
{
  // finding existing TextureSheet
  ntsmap::const_iterator it;
  it = _sheetPool.find(filename);
  if (it != _sheetPool.end())
    return it->second;
  // allocating new TextureSheet
  TextureSheet * new_texture_sheet = nullptr;
  try {
    new_texture_sheet = new TextureSheet(filename, total_frames,
                                         columns, rows);
  }
  catch (const Error & error) { std::cout << error; }
  _sheetPool[filename] = new_texture_sheet;
  return new_texture_sheet;
}

/*****************************************************************************/
/*!
\brief
  Purges and deletes all Textures within the TexturePool.
*/
/*****************************************************************************/
void TexturePool::Purge()
{
  {
    // clearing textures
    ntmap::iterator it = _pool.begin();
    ntmap::iterator it_e = _pool.end();
    for (; it != it_e; ++it) {
      // purge
      try { it->second->Purge(); }
      catch (const Error & error) { std::cout << error; }
      // deallocate
      delete it->second;
    }
  }
  {
    // clearing texture sheets
    ntsmap::iterator it = _sheetPool.begin();
    ntsmap::iterator it_e = _sheetPool.end();
    for (; it != it_e; ++it) {
      // purge
      try { it->second->Purge(); }
      catch (const Error & error) { std::cout << error; }
      // deallocate
      delete it->second;
    }
  }

}