/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file TextureSheet.cpp
\author Connor Deakin
\par E-mail: connor.deakin\@digipen.edu
\par Project: Graphics
\date 07/03/2017
\brief
  Contains the implementation of the TextureSheet. Look in the TextureSheet
  header for more information.
*/
/*****************************************************************************/

#include "OpenGLError.h"

#include "TextureSheet.h"

//! Texture Sheet error thrown from the TextureSheet constructor when an error
// is found.
#define SHEETERROR(message, texture_file)\
Error error("TextureSheet.cpp", "TextureSheet()");\
error.Add(message);\
error.Add("<Texture File>");\
error.Add(texture_file);\
throw(error);

///////////////////////////////////////////////////////////////////////////////
// TEXTURE SHEET                                                             //
///////////////////////////////////////////////////////////////////////////////

/*****************************************************************************/
/*!
\brief
  Creates a TextureSheet that starts with a current frame number of zero.

\param file
  The file containing the texture that is being split by the TextureSheet.
\param total_frames
  The total number of frames on the Texture.
\param rows
  The total number of rows of frames on the Texture.
\param columns
  The total number of columns of frames on the Texture.
*/
/*****************************************************************************/
TextureSheet::TextureSheet(const std::string & file, unsigned int total_frames,
                           unsigned int columns, unsigned int rows) :
Texture(file), _totalFrames(total_frames), _columns(columns), _rows(rows)
{
  // error checking frame information
  unsigned int max_frames = rows * columns;
  unsigned int width_remainder = _width % columns;
  unsigned int height_remainder = _height % rows;
  // un-even width
  if (width_remainder != 0){
    SHEETERROR("The given texture does not split evenly with the number of "
               "columns specified along its width.", _textureFile.c_str())
  }
  // un-even height
  if (height_remainder != 0) {
    SHEETERROR("The given texture does not split evenly with the number of "
               "rows specified along its height.", _textureFile.c_str())
  }
  // too many frames
  if (total_frames > max_frames) {
    SHEETERROR("The given total frames is larger than the number of frames "
               "that can exist on the TextureSheet with the number of "
               "rows and columns specified.", _textureFile.c_str())
  }
  // find aspect ratio for a frame.
  int frame_width = _width / columns;
  int frame_height = _height / rows;
  _frameAspectRation = (float)frame_width / (float)frame_height;
}

/*!
\brief Gets the total number of frames on the TextureSheet.
\return The total number of frames on the TextureSheet.
*/
unsigned int TextureSheet::TotalFrames() const
{
  return _totalFrames;
}

/*!
\brief Gets the number of rows on the TextureSheet.
\return The number of rows on the TextureSheet.
*/
unsigned int TextureSheet::Rows() const
{
  return _rows;
}

/*!
\brief Gets the number of columns on the TextureSheet.
\return The number of columns on the TextureSheet.
*/
unsigned int TextureSheet::Columns() const
{
  return _columns;
}

/*!
\brief Gets the AspectRatio for a single frame on the TextureSheet.
\return The AspectRatio for a single frame of the TextureSheet.
*/
float TextureSheet::FrameAspectRatio() const
{
  return _frameAspectRation;
}
