/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file TextureSheetSprite.cpp
\author Connor Deakin
\par E-mail: connor.deakin\@digipen.edu
\par Project: Graphics
\date 09/03/2017
\brief
  Contains the implementation of the TextureSheetSprite.
*/
/*****************************************************************************/

#include <GLM/glm/gtc/type_ptr.hpp>
#include <GLM/glm/gtc/matrix_transform.hpp>

#include "OpenGLError.h"
#include "GraphicPool.h"

#include "TextureSheetSprite.h"

/*****************************************************************************/
/*!
\brief
  Creates a TextureSheetSprite.

\param shader
  The shader used when drawing the TextureSheetSprite.
\param texture_sheet
  The TextureSheet whose frames will be displayed.
*/
/*****************************************************************************/
TextureSheetSprite::TextureSheetSprite(const ShaderTextureSheet * shader,
                                       const TextureSheet * texture_sheet):
Sprite(TEXTURESHEET), _shader(shader), _textureSheet(texture_sheet),
_currentFrame(0, 0), _currentFrameNumber(0), _isAdditive(false)
{}

/*****************************************************************************/
/*!
\brief
  Will draw the current frame of the TextureSheetSprite to screen using the
  TextureSheetSprite's shader.
*/
/*****************************************************************************/
void TextureSheetSprite::Draw(const glm::mat4 & object_to_ndc) const
{
  // Applying TextureSheet aspect ratio
  glm::vec3 texture_sheet_scale(_textureSheet->FrameAspectRatio(), 1.0f, 1.0f);
  glm::mat4 transformation(glm::scale(object_to_ndc, texture_sheet_scale));
  // Setting the shader uniforms
  _shader->Use();
  glUniformMatrix4fv(_shader->TransformationU(), 1, GL_FALSE, 
                     glm::value_ptr(transformation));
  glUniform4f(_shader->ColorU(), _color.r, _color.g, _color.b, _color.a);
  glUniform2ui(_shader->TotalFramesU(), _textureSheet->Columns(),
               _textureSheet->Rows());
  glUniform2ui(_shader->CurrentFrameU(), _currentFrame.x, _currentFrame.y);
  glUniform1i(_shader->AdditiveU(), static_cast<int>(_isAdditive));
  // error check
  GLenum error_code = glGetError();
  if (error_code) {
    OpenGLError error("TextureSheetSprite.cpp", "Draw");
    error.Code(error_code);
    error.Add("Encountered while setting the ShaderTextureSheet's uniforms.");
    throw(error);
  }
  // draw
  _textureSheet->Bind();
  GraphicPool::Texture().Draw();
}

/*****************************************************************************/
/*!
\brief
  Moves to the next frame on the TextureSheet.

\par Important Notes
  - Will wrap to the first frame after passing the last frame.
*/
/*****************************************************************************/
void TextureSheetSprite::NextFrame()
{
  ++_currentFrameNumber;
  // wrap
  if (_currentFrameNumber >= _textureSheet->TotalFrames()) {
    _currentFrame = glm::tvec2<unsigned int>(0, 0);
    _currentFrameNumber = 0;
    return;
  }
  // continue
  ++_currentFrame.x;
  if (_currentFrame.x >= _textureSheet->Columns()) {
    _currentFrame.x = 0;
    ++_currentFrame.y;
  }
}

/*****************************************************************************/
/*!
\brief
  Moves to the previous frame on the TextureSheet.

\par Important Notes
  - Will wrap to the last frame after passing the first frame.
*/
/*****************************************************************************/
void TextureSheetSprite::PreviousFrame()
{
  // wrap
  --_currentFrameNumber;
  if (_currentFrameNumber >= _textureSheet->TotalFrames()) {
    SetFrameNumber(_textureSheet->TotalFrames() - 1);
    return;
  }
  // continue
  --_currentFrame.x;
  if (_currentFrame.x >= _textureSheet->TotalFrames()) {
    _currentFrame.x = _textureSheet->Columns() - 1;
    --_currentFrame.y;
  }
}

/*****************************************************************************/
/*!
\brief
  Set the location of the current frame on the TextureSheet to draw when
  the Draw function is called.

\param column
  The column of the frame.
\param row
  The row of the frame.
*/
/*****************************************************************************/
void TextureSheetSprite::SetCurrentFrame(unsigned int column, unsigned int row)
{
  // error checking
  if (column >= _textureSheet->Columns()) {
    Error error("TextureSheetSprite.cpp", "SetCurrentFrame");
    error.Add("The column specified does not exist within this Sprite's "
              "TextureSheet");
    throw(error);
  }
  if (row >= _textureSheet->Rows()) {
    Error error("TextureSheetSprite.cpp", "SetCurrentFrame");
    error.Add("The row specified does not exist within this Sprite's " 
              "TextureSheet.");
    throw(error);
  }
  unsigned int new_frame_number = row * _textureSheet->Columns() + column;
  if (new_frame_number >= _textureSheet->TotalFrames()) {
    Error error("TextureSheetSprite.cpp", "SetCurrentFrame");
    error.Add("The specified frame comes after the last frame on this Sprite's"
              "TextureSheet");
    throw(error);
  }
  // setting
  _currentFrame.x = column;
  _currentFrame.y = row;
  _currentFrameNumber = new_frame_number;
}

/*****************************************************************************/
/*!
\brief
  Set the frame of the TextureSheet that is drawn when the TextureSheetSprite
  is drawn.

\param new_frame_number
  The new value of the _currentFrameNumber.
*/
/*****************************************************************************/
void TextureSheetSprite::SetFrameNumber(unsigned int new_frame_number)
{
  // error check
  if (new_frame_number >= _textureSheet->TotalFrames()) {
    Error error("TextureSheet.cpp", "SetFrameNumber");
    error.Add("The specified frame comes after the last frame on this "
      "TextureSheet");
    throw(error);
  }
  // set
  _currentFrame.x = new_frame_number % _textureSheet->Columns();
  _currentFrame.y = new_frame_number / _textureSheet->Columns();
  _currentFrameNumber = new_frame_number;
}

/*!
\brief The sprites color value will be added to the texture color when true.
\param is_additve Whether the sprite color is additive or not.
*/
void TextureSheetSprite::SetAdditive(bool is_additive)
{
  _isAdditive = is_additive;
}

/*!
\brief Gets the TextureSheet that is used by this Sprite.
\return The TextureSheet used by this Sprite.
*/
const TextureSheet & TextureSheetSprite::Sheet()
{
  return *_textureSheet;
}