/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file TextureSprite.cpp
\author Connor Deakin
\par E-mail: connor.deakin\@digipen.edu
\par Project: Graphics
\date 09/03/2017
\brief
  Contains the implementation of the TextureSprite.
*/
/*****************************************************************************/

#include <GLM/glm/gtc/type_ptr.hpp>
#include <GLM/glm/gtc/matrix_transform.hpp>

#include "OpenGLError.h"
#include "GraphicPool.h"

#include "TextureSprite.h"

/*****************************************************************************/
/*!
\brief
  Creates a TextureSprite.

\param shader
  The shader used when drawing the TextureSprite.
\param texture
  The texture that is displayed.
*/
/*****************************************************************************/
TextureSprite::TextureSprite(const ShaderTexture * shader,
                             const Texture * texture) :
Sprite(TEXTURE), _shader(shader), _texture(texture), _isAdditive(false)
{}

/*****************************************************************************/
/*!
\brief
  Draws the TextureSprite to screen using the shader given to the
  TextureSprite.
*/
/*****************************************************************************/
void TextureSprite::Draw(const glm::mat4 & object_to_ndc) const
{
  // apply texture aspect ratio
  glm::vec3 texture_scale(_texture->AspectRatio(), 1.0f, 1.0f);
  glm::mat4 transformation(glm::scale(object_to_ndc, texture_scale));
  // setting uniforms
  _shader->Use();
  glUniformMatrix4fv(_shader->TransformationU(), 1, GL_FALSE, 
                     glm::value_ptr(transformation));
  glUniform4fv(_shader->ColorU(), 1, glm::value_ptr(_color));
  glUniform1i(_shader->AdditiveU(), static_cast<int>(_isAdditive));
  // error check
  GLenum error_code = glGetError();
  if (error_code) {
    OpenGLError error("TextureSprite.cpp", "Draw");
    error.Code(error_code);
    error.Add("Encountered while setting the "
              "ShaderTexture's color uniform.");
    throw(error);
  }
  // drawing
  _texture->Bind();
  GraphicPool::Texture().Draw();
}

/*****************************************************************************/
/*!
\brief
  Set the Sprite's texture to a different texture.

\param new_texture
  The Sprite's new texture.
*/
/*****************************************************************************/
void TextureSprite::SetTexture(const Texture * new_texture)
{
  _texture = new_texture;
}

/*!
\brief Set whether the sprite color is additive of multiplicative.
\param is_additive If true, the color will be additive.
*/
void TextureSprite::SetAdditive(bool is_additive)
{
  _isAdditive = is_additive;
}
