/* All content(c) 2016 - 2017 DigiPen(USA) Corporation, all rights reserved. */
/*****************************************************************************/
/*!
\file Transition.cpp
\author Connor Deakin
\par E-mail: connor.deakin\@digipen.edu
\par Project: X-Craft
\date 15/04/2017
\brief
  Contains the implementation of the Transition state.
*/
/*****************************************************************************/

#include "Space.h"
#include "Time.h"
#include "AudioEngine.h"
#include "SpritePool.h"
#include "ShaderPool.h"
#include "SpaceManager.h"

#include "Transition.h"

/*****************************************************************************/
/*!
\brief
  Creates the Transition state.

\param transition_info
  Information about the transition that is about to take place.
*/
/*****************************************************************************/
Transition::Transition(const TransitionInfo & transition_info):
  _info(transition_info), _currentTime(0.0f), _currentState(FADEIN), _coverT(),
  _levelLoaded(false)
{}

/*****************************************************************************/
/*!
\brief
  Prepares the transition.
*/
/*****************************************************************************/
void Transition::Allocate()
{
  // add a remove function to the sprite pool
  _coverSprite = SpritePool::AddGraphic(&ShaderPool::Graphic());
  (**_space).GetRenderer().Add(_coverSprite, &_coverT);
  _coverSprite->SetColor(0.0f, 0.0f, 0.0f, 0.0f);
  _coverT.Scale() = 10.0f;

  Object * loading_symbol;
  loading_symbol = _objectManager.CreateArchetypeObjectAtPosition("loading",
    glm::vec3(0.0f, 0.0f, 0.0f));
  loading_symbol->GetTransform()->Scale() = 0.25f;
  _loadingSprite = loading_symbol->GetSprite();
  _loadingSprite->SetAlpha(0.0f);

  _startVolume = audEngine.GetVolumeAllChannels();
}

/*****************************************************************************/
/*!
\brief
  Performs smooth level transition.
/*****************************************************************************/
bool Transition::Update()
{
  _currentTime += Time::DT();
  float fade_percentage;
  switch (_currentState)
  {
  case FADEIN:
    // increase alpha
    fade_percentage = _currentTime / _info._fadeTime;
    if (_currentTime > _info._fadeTime) {
      // deleting the old space
      _coverSprite->SetAlpha(1.0f);
      _loadingSprite->SetAlpha(1.0f);
      Global::SpaceManager::RemoveAllBeneath(_space);
      audEngine.SetVolumeAllChannels(0.0f);
      _currentState = STILL;
      _currentTime = 0.0f;
    }
    else {
      _coverSprite->SetAlpha(fade_percentage);
      _loadingSprite->SetAlpha(fade_percentage);
      audEngine.SetVolumeAllChannels((1.0f - fade_percentage) * _startVolume);
    }
    break;
  case STILL:
    // waiting till rest time has passed
    if (_currentTime > _info._restTime) {
      _currentState = FADEOUT;
      _currentTime = 0.0f;
    }
    break;
  case FADEOUT:
    // load level at start of rest
    if (!_levelLoaded) {
      if (!_info._levelFile.empty())
        Global::SpaceManager::AddBottom(_info._levelFile, 30.0f);
      else
        Global::SpaceManager::AddBottom(_info._nextState, 30.0f);
      _levelLoaded = true;
      _currentTime = 0.0f;
    }
    // lightening alpha
    fade_percentage = _currentTime / _info._fadeTime;
    _coverSprite->SetAlpha(1.0f - fade_percentage);
    _loadingSprite->SetAlpha(1.0f - fade_percentage);
    audEngine.SetVolumeAllChannels(fade_percentage * _startVolume);
    // destroying this space
    if (_currentTime > _info._fadeTime) {
      audEngine.SetVolumeAllChannels(_startVolume);
      Global::SpaceManager::Remove(_space);
    }
    break;
  }
  return true;
}

/*****************************************************************************/
/*!
\brief
  Needs to remove the graphic sprite from the sprite pool.
*/
/*****************************************************************************/
void Transition::Free()
{
}